package io.paulik.mqtt.client;

import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.concurrent.TimeUnit;

/**
 * Created by apaulik on 5/1/17.
 */
public class UseCase2MqttClient {

    public static void main(String[] args) throws MqttException, UnsupportedEncodingException, InterruptedException{
        Path path0 = Paths.get("uc1start.txt");
        Path path1 = Paths.get("uc1end.txt");
        final BufferedWriter writer, writer2;
        try {
            writer = Files.newBufferedWriter(path0);
            writer2 = Files.newBufferedWriter(path1);

            MqttClient client = new MqttClient(
                    "tcp://localhost:1883", //URI
                    MqttClient.generateClientId(), //ClientId
                    new MemoryPersistence()); //Persistence


            MqttConnectOptions options = new MqttConnectOptions();
            options.setMqttVersion(MqttConnectOptions.MQTT_VERSION_3_1_1);
            options.setUserName("admin");
            options.setPassword("123456789Admin".toCharArray());
            client.connect(options);

            client.setCallback(new MqttCallback() {

                public void connectionLost(Throwable cause) { //Called when the client lost the connection to the broker
                }

                public void messageArrived(String topic, MqttMessage message) throws Exception {
                    //System.out.println(topic + ": " + new String(message.getPayload()));
                    writer2.write(System.currentTimeMillis() + "\n");

                }

                public void deliveryComplete(IMqttDeliveryToken token) {//Called when a outgoing publish is complete
                }
            });

            client.subscribe("/response/bp1750/readLuminanceNormalized", 2);

            for (int i = 0; i < 1200; i++){
                writer.write(System.currentTimeMillis() + "\n");
                client.publish(
                        "/request/bp1750/readLuminanceNormalized", // topic
                        "".getBytes("UTF-8"), // payload
                        2, // QoS
                        false); // retained?
                TimeUnit.SECONDS.sleep(3);
            }

            client.disconnect();

            writer.close();
            writer2.close();

        }catch (IOException e){}
    }
}
