package io.paulik.camel.rest;

import io.silverspoon.bulldog.core.io.bus.i2c.I2cBus;
import io.silverspoon.bulldog.core.platform.Board;
import io.silverspoon.bulldog.core.platform.Platform;
import io.silverspoon.bulldog.devices.sensors.BH1750LightIntensitySensor;
import io.silverspoon.bulldog.raspberrypi.RaspiNames;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.apache.camel.builder.RouteBuilder;

/**
 * Created by apaulik on 4/21/17.
 */
public class UseCase2RestBuilder extends RouteBuilder {
    public void configure() throws Exception {

        restConfiguration().component("restlet").host("localhost").port(8080); //.bindingMode(RestBindingMode.auto)

        rest("/")
                .get("/bh1750/readLuminanceNormalized").to("direct:bh1750");

        from("direct:bh1750").process(new Processor() {
            public void process(Exchange exchange) throws Exception {
                exchange.getOut().setHeader(Exchange.HTTP_RESPONSE_CODE, 200);
                exchange.getOut().setHeader(Exchange.CONTENT_TYPE, "text/html");
                // TODO: 4/21/17 read data with bulldog
                //Detect the board we are running on
                Board board = Platform.createBoard();

                //Set up a digital output
                //DigitalOutput output = board.getPin(RaspiNames.P1_11).as(DigitalOutput.class);

                //Blink the LED
                //output.high();
                //BulldogUtil.sleepMs(1000);
                //output.low();

                I2cBus bus = board.getI2cBus(RaspiNames.I2C_1);

                // Let's assume we have got a device on address xx
                BH1750LightIntensitySensor sensor = new BH1750LightIntensitySensor(bus, 0x23);
                sensor.initMode(BH1750LightIntensitySensor.MODE_HIGH_RES_05_LX_CONTINUOUS);



                double luminosity = sensor.readLuminanceNormalized();
                exchange.getOut().setBody("" + luminosity);
            }
        });
    }
}
